/**
 * @file FaspManagerSample.h
 * @author Aspera, Inc. 2005-2009
 */

#ifndef __FASP_MANAGER_SAMPLE_H__
#define __FASP_MANAGER_SAMPLE_H__

#ifdef WIN32
#include <winsock.h>
#endif

#include <sys/types.h>
#include <stdio.h>
#include "FaspManagerLocation.h"
#include "FaspManagerTypes.h"
#include "FaspManager.h"
#include "Environment.h"

/**
 * A class to receive call backs from the Aspera Fasp Manager 
 */
class MyCallbackClass
{
private:
	bool m_sessionReady;
	int m_fileTransferCount;

  /* Member functions */
public:

/**
 * Constructs this call back class.
 */
MyCallbackClass( )
{
	m_sessionReady = false;
	m_fileTransferCount = 0;
}

/**
 * Returns true if the session has started, false if not or ended
 */
bool isSessionReady() {
	return m_sessionReady;
}


/**
 * Returns how many files have been transferred in this session
 */
int getFileTransferCount() {
	return m_fileTransferCount;
}


/**
 * We just needed a cross-platform sleep() function... this fit the bill
 */
void sleep(long milliseconds) {
	struct timeval tv;
	tv.tv_sec = milliseconds / 1000;
	tv.tv_usec = (milliseconds % 1000) * 1000;
	select(0, NULL, NULL, NULL, &tv);
}

/**
 * Receives callbacks from the FASP Manager for notification of statistics
 * pertaining to the FASP Job and its current file. Note that this signature 
 * is invariant to receive such notification.
 *
 * @param j_stats the FASP Job job-level statistics
 * @param f_stats the FASP Job file-level statistics for the current file
 */

void ReceiveCallback(Aspera::FaspManager::callback_event_e cb_event, const Aspera::FaspManager::JobStats *j_stats, const Aspera::FaspManager::FileStats *f_stats )
{
	switch(cb_event) {
		case Aspera::FaspManager::F_EVENT_BANDWIDTH_MEASUREMENT :
	        printf("Measuring initial bandwidth...\n");
			break;
		case Aspera::FaspManager::F_EVENT_CONNECTING :
			printf("Connecting...\n");
			break;
		case Aspera::FaspManager::F_EVENT_SESSION_START :
			m_sessionReady = true;		//  Session has started
	        printf("Session started. ID: %s\n",j_stats->job_id);
			break;
	    case Aspera::FaspManager::F_EVENT_FILE_START :
	        printf("File: %s started.\n",f_stats->filename);
			break;
		case Aspera::FaspManager::F_EVENT_PROGRESS :
			printf("Progress: %llu/%llu bytes\n", f_stats->bytes_written, f_stats->file_size);
			break;	
		case Aspera::FaspManager::F_EVENT_RATE_MODIFICATION :
	        printf("Target transfer rate modified to: %llu KBps\n", j_stats->target_rate);
			break;					
	    case Aspera::FaspManager::F_EVENT_FILE_STOP :
			m_fileTransferCount += 1;
	        printf("File: %s completed.\n",f_stats->filename);
			break;
		case Aspera::FaspManager::F_EVENT_SESSION_STOP :
			m_sessionReady = false;		//  Normal session end
			printf("Session stopped ID: %s\n", j_stats->job_id);
			break;
		case Aspera::FaspManager::F_EVENT_FILE_ERROR :
			m_fileTransferCount += 1;
	        printf("Error on file : %s  Reason: %s\n",f_stats->filename,f_stats->err_description);
			break;
		case Aspera::FaspManager::F_EVENT_SESSION_ERROR :
			m_sessionReady = false;		//  Session aborted abnormally
	        printf("Error in session ID: %s  Reason: %s\n",j_stats->job_id,j_stats->err_description);
			break;					
		default:
			printf("Unknown event received: %d\n", cb_event);

		}
}

};

#endif /* __FASP_MANAGER_SAMPLE_H__ */
